#!perl -w
use strict;
use Test::More tests => 15;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw (RunOnStore GetTestSite);
use DE_EPAGES::Object::DAL::UnitOfMeasurement;

use utf8;

use constant GRAM_CODE => 'GRM';
use constant GRAM_ABBREVIATION => 'g';
use constant MILIMETER_CODE => 'MMT';
use constant MILIMETER_ABBREVIATION => 'mm';
use constant KILOGRAM_CODE => 'KGM';
use constant KILOGRAM_ABBREVIATION => 'kg';
use constant SQUARECENTIMETER_CODE => 'CMK';
use constant SQUARECENTIMETER_ABBREVIATION => 'cm²';
use constant INCORRECT_SQUAREMETER_ABBREVIATION => 'sqm';

sub Main {
    ## no critic qw(ProhibitMagicNumbers)
    ok( Init(), 'initialization' );

    my $SubTestName = (caller(0))[3];

    my $ahResulPkeyGram = DAL_GetPKeyUnitOfMeasurementByCode(GRAM_CODE);
    is(scalar @{$ahResulPkeyGram}, 1, "$SubTestName - test one single primary key for gram gotten by code");
    my $hGramPKey = $ahResulPkeyGram->[0];

    my $ahResultInfoGram = DAL_GetInfoUnitOfMeasurementByCode(GRAM_CODE);
    is(scalar @{$ahResultInfoGram}, 1, "$SubTestName - test one single information entry for gram gotten by code");
    my $hGramInfo = $ahResultInfoGram->[0];
    is_deeply($hGramInfo,
            {
                'UnitID' => $hGramPKey->{'UnitID'},
                'Type' => 'mass',
                'Factor' => undef,
                'Abbreviation' => GRAM_ABBREVIATION,
                'Code' => GRAM_CODE
            },
            "$SubTestName - test gram information");

    my $ahResulPkeyMilimeter = DAL_GetPKeyUnitOfMeasurementByCode(MILIMETER_CODE);
    is(scalar @{$ahResulPkeyMilimeter}, 1, "$SubTestName - test one single primary key for milimeter gotten by code");
    my $hMilimeterPKey = $ahResulPkeyMilimeter->[0];

    my $ahResultInfoMilimeter = DAL_GetInfoUnitOfMeasurementByCode(MILIMETER_CODE);
    is(scalar @{$ahResultInfoMilimeter}, 1, "$SubTestName - test one single information entry for milimeter gotten by code");
    my $hMilimeterInfo = $ahResultInfoMilimeter->[0];
    is_deeply($hMilimeterInfo,
            {
                'UnitID' => $hMilimeterPKey->{'UnitID'},
                'Type' => 'length',
                'Factor' => 0.001,
                'Abbreviation' => MILIMETER_ABBREVIATION,
                'Code' => MILIMETER_CODE
            },
            "$SubTestName - test milimeter information");

    my $ahResulPkeyKilogram = DAL_GetPKeyUnitOfMeasurementByAbbreviation(KILOGRAM_ABBREVIATION);
    is(scalar @{$ahResulPkeyKilogram}, 1, "$SubTestName - test one single primary key for kilogram gotten by abbreviation");
    my $hKilogramPKey = $ahResulPkeyKilogram->[0];

    my $ahResultInfoKilogram = DAL_GetInfoUnitOfMeasurementByAbbreviation(KILOGRAM_ABBREVIATION);
    is(scalar @{$ahResultInfoKilogram}, 1, "$SubTestName - test one single information entry for kilogram gotten by abbreviation");
    my $hKilogramInfo = $ahResultInfoKilogram->[0];
    is_deeply($hKilogramInfo,
            {
                'UnitID' => $hKilogramPKey->{'UnitID'},
                'Type' => 'mass',
                'Factor' => 1000,
                'Abbreviation' => KILOGRAM_ABBREVIATION,
                'Code' => KILOGRAM_CODE
            },
            "$SubTestName - test kilogram information");

    my $ahResulPkeySquarecentimeter = DAL_GetPKeyUnitOfMeasurementByAbbreviation(SQUARECENTIMETER_ABBREVIATION);
    is(scalar @{$ahResulPkeySquarecentimeter}, 1, "$SubTestName - test one single primary key for square centimeter gotten by abbreviation");
    my $hSquarecentimeterPKey = $ahResulPkeySquarecentimeter->[0];

    my $ahResultInfoSquarecentimeter = DAL_GetInfoUnitOfMeasurementByAbbreviation(SQUARECENTIMETER_ABBREVIATION);
    is(scalar @{$ahResultInfoSquarecentimeter}, 1, "$SubTestName - test one single information entry for square centimeter gotten by abbreviation");
    my $hSquarecentimeterInfo = $ahResultInfoSquarecentimeter->[0];
    is_deeply($hSquarecentimeterInfo,
            {
                'UnitID' => $hSquarecentimeterPKey->{'UnitID'},
                'Type' => 'area',
                'Factor' => 0.0001,
                'Abbreviation' => SQUARECENTIMETER_ABBREVIATION,
                'Code' => SQUARECENTIMETER_CODE
            },
            "$SubTestName - test square centimeter information");

    my $ahResulPkeySqm = DAL_GetPKeyUnitOfMeasurementByAbbreviation(INCORRECT_SQUAREMETER_ABBREVIATION);
    is(scalar @{$ahResulPkeySqm}, 0, "$SubTestName - test no primary key found for incorrect square meter abbreviation code");

    ok( Cleanup(), 'Cleanup' );
}

sub Cleanup {
    return 1;
}

sub Init {
    Cleanup();

    return 1;
}

RunScript( Message => 'long', Sub=> sub {
    RunOnStore( Store => GetTestSite(), Sub => \&Main );
});
