#========================================================================================
# package      DE_EPAGES::PayPalPlus::t::unit::UI_Shop_PaymentBacklinkPayPalPlus
# base         DE_EPAGES::Test::API::UnitTest::Test
#----------------------------------------------------------------------------------------
# description  unit test for DE_EPAGES::PayPalPlus::UI::Shop
#========================================================================================
package DE_EPAGES::PayPalPlus::t::unit::UI_Shop_PaymentBacklinkPayPalPlus;
use base DE_EPAGES::Test::API::UnitTest::Test;

use DE_EPAGES::Test::Mock::Mockify qw (
    WasCalled
    GetParametersFromMockifyCall
    GetCallCount
);
use DE_EPAGES::Test::Mock::MockifyServlet;
use DE_EPAGES::PayPalPlus::API::Constants qw (
    MULTISTEP_CHECKOUT
    CLASSIC_CHECKOUT
);
use Test::More;
use Test::Exception;
use strict;

sub test {
    my $self = shift;

    $self->_prepareMocks();

    $self->test_PaymentBacklinkPayPalPlus_success();

    $self->test_PaymentBacklinkPayPalPlus_failure_multistep_before();
    $self->test_PaymentBacklinkPayPalPlus_failure_multistep_after();

    $self->test_PaymentBacklinkPayPalPlus_failure_classical_before();
    $self->test_PaymentBacklinkPayPalPlus_failure_classical_after();

    $self->test_PaymentBacklinkPayPalPlus_failure_mobile_before();
    $self->test_PaymentBacklinkPayPalPlus_failure_mobile_after();

    $self->test_PaymentBacklinkPayPalPlus_failure_unity_before();

    done_testing();
}

sub test_PaymentBacklinkPayPalPlus_success {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    ## GIVEN
    my $SUT = $self->_buildSUT({
        'OrderBeforePayment' => 1,
        'ID' => '20304',
        'Shop' => $self->memory->get('MultistepShop'),
        'CanProcessOrder' => 1
    });
    my $Servlet = $self->_buildServlet({
        'IsMobile' => 1,
        'Shop' => $self->memory->get('MultistepShop'),
        'PaymentLineItemGUID' => '445566-332211-221144',
        'ObjectID' => '12233',
        'DereferrerToken' => 'xGHPdGpZ'
    });

    ## WHEN
    $SUT->PaymentBacklinkPayPalPlus($Servlet);

    ## THEN
    ok(WasCalled($self->memory->get('LogContext'), 'logContext'),
        "$SubTestName - verify logContext was used.");
    ok(WasCalled($self->memory->get('Logger'), 'logPayment'),
        "$SubTestName - verify logPayment was used.");
    ok(WasCalled($SUT, 'BlockMonitor'),
        "$SubTestName - verify block monitor was used.");

    ok(WasCalled($Servlet, 'locale'),
        "$SubTestName - verify locale was reseted.");
    ok(WasCalled($Servlet, 'languageID'),
        "$SubTestName - verify languageID was reseted.");
    ok(WasCalled($Servlet, 'currencyID'),
        "$SubTestName - verify currencyID was reseted.");

    my $aParameters = GetParametersFromMockifyCall($SUT, '_ValidateDereferrerToken');
    my ($Token, $PayLoad) = @{$aParameters};

    is($Token, 'xGHPdGpZ',
        "$SubTestName - verify token was params was used for request validation.");
    is($PayLoad, 'MultistepShop&12233&445566-332211-221144',
        "$SubTestName - verify correct payload was used for request validation.");

    is($Servlet->vars('WasSuccessful'), 1,
        "$SubTestName - verify servlet var WasSuccessful was set accordingly.");
    is($Servlet->vars('ViewObjectID'), $self->memory->get('CustomerOrder')->id(),
        "$SubTestName - verify servlet var ViewObjectID was set accordingly.");

    return;
}

sub test_PaymentBacklinkPayPalPlus_failure_multistep_before {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    ## GIVEN
    my $SUT = $self->_buildSUT({
        'OrderBeforePayment' => 1,
        'ID' => '20304',
        'Shop' => $self->memory->get('MultistepShop'),
        'CanProcessOrder' => 0
    });
    my $Servlet = $self->_buildServlet({
        'IsMobile' => 0,
        'Shop' => $self->memory->get('MultistepShop'),
        'PaymentLineItemGUID' => '445566-332211-221144',
        'ObjectID' => '12233',
        'DereferrerToken' => 'xGHPdGpZ'
    });

    ## WHEN
    $SUT->PaymentBacklinkPayPalPlus($Servlet);

    ## THEN
    ok(WasCalled($self->memory->get('LogContext'), 'logContext'),
        "$SubTestName - verify logContext was used.");
    ok(WasCalled($self->memory->get('Logger'), 'logPayment'),
        "$SubTestName - verify logPayment was used.");
    ok(WasCalled($SUT, 'BlockMonitor'),
        "$SubTestName - verify block monitor was used.");

    ok(WasCalled($Servlet, 'locale'),
        "$SubTestName - verify locale was reseted.");
    ok(WasCalled($Servlet, 'languageID'),
        "$SubTestName - verify languageID was reseted.");
    ok(WasCalled($Servlet, 'currencyID'),
        "$SubTestName - verify currencyID was reseted.");

    ok(WasCalled($Servlet->form(), 'executeFormError'),
        "$SubTestName - verify form error was executed.");
    is_deeply(GetParametersFromMockifyCall($Servlet->form(), 'executeFormError')->[0], {
        'Name'        => 'PayPalPlus_PaymentFailed',
        'Reason'      => 'FailedToProcessOrder',
        'ViewAction'  => 'ViewPaymentPayPalPlusResult',
        'ObjectID'    => '20304'
    }, "$SubTestName - verify the form error has expected attributes.");

    return;
}

sub test_PaymentBacklinkPayPalPlus_failure_multistep_after {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    ## GIVEN
    my $SUT = $self->_buildSUT({
        'OrderBeforePayment' => 0,
        'ID' => '20304',
        'Shop' => $self->memory->get('MultistepShop'),
        'CanProcessOrder' => 0
    });
    my $Servlet = $self->_buildServlet({
        'IsMobile' => 0,
        'Shop' => $self->memory->get('MultistepShop'),
        'PaymentLineItemGUID' => '445566-332211-221144',
        'ObjectID' => '12233',
        'DereferrerToken' => 'xGHPdGpZ'
    });

    ## WHEN
    $SUT->PaymentBacklinkPayPalPlus($Servlet);

    ## THEN
    ok(WasCalled($self->memory->get('LogContext'), 'logContext'),
        "$SubTestName - verify logContext was used.");
    ok(WasCalled($self->memory->get('Logger'), 'logPayment'),
        "$SubTestName - verify logPayment was used.");
    ok(WasCalled($SUT, 'BlockMonitor'),
        "$SubTestName - verify block monitor was used.");

    ok(WasCalled($Servlet, 'locale'),
        "$SubTestName - verify locale was reseted.");
    ok(WasCalled($Servlet, 'languageID'),
        "$SubTestName - verify languageID was reseted.");
    ok(WasCalled($Servlet, 'currencyID'),
        "$SubTestName - verify currencyID was reseted.");

    my $aValidateDereferrerTokenParams = GetParametersFromMockifyCall($SUT, '_ValidateDereferrerToken');
    my ($Token, $PayLoad) = @{$aValidateDereferrerTokenParams};

    is($Token, 'xGHPdGpZ',
        "$SubTestName - verify token was params was used for request validation.");
    is($PayLoad, 'MultistepShop&12233&445566-332211-221144',
        "$SubTestName - verify correct payload was used for request validation.");

    my $aBuildShopUrlParams = GetParametersFromMockifyCall($SUT, '_BuildShopUrl');
    my ($Object, $hParams, $hOptions) = @{$aBuildShopUrlParams}; ## no critic (ProhibitUnusedVariablesEnhanced)

    is_deeply($hParams, {
        'ViewAction' => 'ViewMultiCheckoutPayment'
    }, "$SubTestName - verify the expected params where used to build the redirect url.");
    is_deeply($hOptions, {
        'Type' => 'sf/mobile',
        'UseSSL' => 1,
        'AddToken' => 1
    }, "$SubTestName - verify the expected options where used to build the redirect url.");
    ok(WasCalled($Servlet, 'redirect'),
        "$SubTestName - verify the servlet redirect was called.");

    is(GetCallCount($Servlet->form(), 'executeFormError'), 0,
        "$SubTestName - verify no form error was executed.");

    return;
}

sub test_PaymentBacklinkPayPalPlus_failure_classical_before {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    ## GIVEN
    my $SUT = $self->_buildSUT({
        'OrderBeforePayment' => 1,
        'ID' => '20304',
        'Shop' => $self->memory->get('ClassicalShop'),
        'CanProcessOrder' => 0
    });
    my $Servlet = $self->_buildServlet({
        'IsMobile' => 0,
        'Shop' => $self->memory->get('ClassicalShop'),
        'PaymentLineItemGUID' => '445566-332211-221144',
        'ObjectID' => '12233',
        'DereferrerToken' => 'xGHPdGpZ'
    });

    ## WHEN
    $SUT->PaymentBacklinkPayPalPlus($Servlet);

    ## THEN
    ok(WasCalled($self->memory->get('LogContext'), 'logContext'),
        "$SubTestName - verify logContext was used.");
    ok(WasCalled($self->memory->get('Logger'), 'logPayment'),
        "$SubTestName - verify logPayment was used.");
    ok(WasCalled($SUT, 'BlockMonitor'),
        "$SubTestName - verify block monitor was used.");

    ok(WasCalled($Servlet, 'locale'),
        "$SubTestName - verify locale was reseted.");
    ok(WasCalled($Servlet, 'languageID'),
        "$SubTestName - verify languageID was reseted.");
    ok(WasCalled($Servlet, 'currencyID'),
        "$SubTestName - verify currencyID was reseted.");

    ok(WasCalled($Servlet->form(), 'executeFormError'),
        "$SubTestName - verify form error was executed.");
    is_deeply(GetParametersFromMockifyCall($Servlet->form(), 'executeFormError')->[0], {
        'Name'        => 'PayPalPlus_PaymentFailed',
        'Reason'      => 'FailedToProcessOrder',
        'ViewAction'  => 'ViewPaymentPayPalPlusResult',
        'ObjectID'    => '20304'
    }, "$SubTestName - verify the form error has expected attributes.");

    return;
}

sub test_PaymentBacklinkPayPalPlus_failure_classical_after {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    ## GIVEN
    my $SUT = $self->_buildSUT({
        'OrderBeforePayment' => 0,
        'ID' => '20304',
        'Shop' => $self->memory->get('ClassicalShop'),
        'CanProcessOrder' => 0
    });
    my $Servlet = $self->_buildServlet({
        'IsMobile' => 0,
        'Shop' => $self->memory->get('ClassicalShop'),
        'PaymentLineItemGUID' => '445566-332211-221144',
        'ObjectID' => '12233',
        'DereferrerToken' => 'xGHPdGpZ'
    });

    ## WHEN
    $SUT->PaymentBacklinkPayPalPlus($Servlet);

    ## THEN
    ok(WasCalled($self->memory->get('LogContext'), 'logContext'),
        "$SubTestName - verify logContext was used.");
    ok(WasCalled($self->memory->get('Logger'), 'logPayment'),
        "$SubTestName - verify logPayment was used.");
    ok(WasCalled($SUT, 'BlockMonitor'),
        "$SubTestName - verify block monitor was used.");

    ok(WasCalled($Servlet, 'locale'),
        "$SubTestName - verify locale was reseted.");
    ok(WasCalled($Servlet, 'languageID'),
        "$SubTestName - verify languageID was reseted.");
    ok(WasCalled($Servlet, 'currencyID'),
        "$SubTestName - verify currencyID was reseted.");

    my $aValidateDereferrerTokenParams = GetParametersFromMockifyCall($SUT, '_ValidateDereferrerToken');
    my ($Token, $PayLoad) = @{$aValidateDereferrerTokenParams};

    is($Token, 'xGHPdGpZ',
        "$SubTestName - verify token was params was used for request validation.");
    is($PayLoad, 'ClassicalShop&12233&445566-332211-221144',
        "$SubTestName - verify correct payload was used for request validation.");

    my $aBuildShopUrlParams = GetParametersFromMockifyCall($SUT, '_BuildShopUrl');
    my ($Object, $hParams, $hOptions) = @{$aBuildShopUrlParams}; ## no critic (ProhibitUnusedVariablesEnhanced)

    is_deeply($hParams, {},
        "$SubTestName - verify the expected params where used to build the redirect url.");
    is_deeply($hOptions, {
        'Type' => 'sf/mobile',
        'UseSSL' => 1,
        'AddToken' => 1
    }, "$SubTestName - verify the expected options where used to build the redirect url.");
    ok(WasCalled($Servlet, 'redirect'),
        "$SubTestName - verify the servlet redirect was called.");

    is(GetCallCount($Servlet->form(), 'executeFormError'), 0,
        "$SubTestName - verify no form error was executed.");

    return;
}

sub test_PaymentBacklinkPayPalPlus_failure_mobile_before {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    ## GIVEN
    my $SUT = $self->_buildSUT({
        'OrderBeforePayment' => 1,
        'ID' => '20304',
        'Shop' => $self->memory->get('MultistepShop'),
        'CanProcessOrder' => 0
    });
    my $Servlet = $self->_buildServlet({
        'IsMobile' => 1,
        'Shop' => $self->memory->get('MultistepShop'),
        'PaymentLineItemGUID' => '445566-332211-221144',
        'ObjectID' => '12233',
        'DereferrerToken' => 'xGHPdGpZ'
    });

    ## WHEN
    $SUT->PaymentBacklinkPayPalPlus($Servlet);

    ## THEN
    ok(WasCalled($self->memory->get('LogContext'), 'logContext'),
        "$SubTestName - verify logContext was used.");
    ok(WasCalled($self->memory->get('Logger'), 'logPayment'),
        "$SubTestName - verify logPayment was used.");
    ok(WasCalled($SUT, 'BlockMonitor'),
        "$SubTestName - verify block monitor was used.");

    ok(WasCalled($Servlet, 'locale'),
        "$SubTestName - verify locale was reseted.");
    ok(WasCalled($Servlet, 'languageID'),
        "$SubTestName - verify languageID was reseted.");
    ok(WasCalled($Servlet, 'currencyID'),
        "$SubTestName - verify currencyID was reseted.");

    ok(WasCalled($Servlet->form(), 'executeFormError'),
        "$SubTestName - verify form error was executed.");
    is_deeply(GetParametersFromMockifyCall($Servlet->form(), 'executeFormError')->[0], {
        'Name'        => 'PayPalPlus_PaymentFailed',
        'Reason'      => 'FailedToProcessOrder',
        'ViewAction'  => 'ViewPaymentPayPalPlusResult',
        'ObjectID'    => '20304'
    }, "$SubTestName - verify the form error has expected attributes.");

    return;
}

sub test_PaymentBacklinkPayPalPlus_failure_mobile_after {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    ## GIVEN
    my $SUT = $self->_buildSUT({
        'OrderBeforePayment' => 0,
        'ID' => '20304',
        'Shop' => $self->memory->get('MultistepShop'),
        'CanProcessOrder' => 0
    });
    my $Servlet = $self->_buildServlet({
        'IsMobile' => 1,
        'Shop' => $self->memory->get('MultistepShop'),
        'PaymentLineItemGUID' => '445566-332211-221144',
        'ObjectID' => '12233',
        'DereferrerToken' => 'xGHPdGpZ'
    });

    ## WHEN
    $SUT->PaymentBacklinkPayPalPlus($Servlet);

    ## THEN
    ok(WasCalled($self->memory->get('LogContext'), 'logContext'),
        "$SubTestName - verify logContext was used.");
    ok(WasCalled($self->memory->get('Logger'), 'logPayment'),
        "$SubTestName - verify logPayment was used.");
    ok(WasCalled($SUT, 'BlockMonitor'),
        "$SubTestName - verify block monitor was used.");

    ok(WasCalled($Servlet, 'locale'),
        "$SubTestName - verify locale was reseted.");
    ok(WasCalled($Servlet, 'languageID'),
        "$SubTestName - verify languageID was reseted.");
    ok(WasCalled($Servlet, 'currencyID'),
        "$SubTestName - verify currencyID was reseted.");

    my $aValidateDereferrerTokenParams = GetParametersFromMockifyCall($SUT, '_ValidateDereferrerToken');
    my ($Token, $PayLoad) = @{$aValidateDereferrerTokenParams};

    is($Token, 'xGHPdGpZ',
        "$SubTestName - verify token was params was used for request validation.");
    is($PayLoad, 'MultistepShop&12233&445566-332211-221144',
        "$SubTestName - verify correct payload was used for request validation.");

    my $aBuildShopUrlParams = GetParametersFromMockifyCall($SUT, '_BuildShopUrl');
    my ($Object, $hParams, $hOptions) = @{$aBuildShopUrlParams}; ## no critic (ProhibitUnusedVariablesEnhanced)

    is_deeply($hParams, {},
        "$SubTestName - verify the expected params where used to build the redirect url.");
    is_deeply($hOptions, {
        'Type' => 'sf/mobile',
        'UseSSL' => 1,
        'AddToken' => 1
    }, "$SubTestName - verify the expected options where used to build the redirect url.");
    ok(WasCalled($Servlet, 'redirect'),
        "$SubTestName - verify the servlet redirect was called.");

    is(GetCallCount($Servlet->form(), 'executeFormError'), 0,
        "$SubTestName - verify no form error was executed.");

    return;
}

sub test_PaymentBacklinkPayPalPlus_failure_unity_before {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    ## GIVEN
    my $SUT = $self->_buildSUT({
        'OrderBeforePayment' => 1,
        'ID' => '20304',
        'Shop' => $self->memory->get('UnityShop'),
        'CanProcessOrder' => 0
    });
    my $Servlet = $self->_buildServlet({
        'IsMobile' => 0,
        'Shop' => $self->memory->get('UnityShop'),
        'PaymentLineItemGUID' => '445566-332211-221144',
        'ObjectID' => '12233',
        'DereferrerToken' => 'xGHPdGpZ'
    });

    ## WHEN
    $SUT->PaymentBacklinkPayPalPlus($Servlet);

    ## THEN
    ok(WasCalled($self->memory->get('LogContext'), 'logContext'),
        "$SubTestName - verify logContext was used.");
    ok(WasCalled($self->memory->get('Logger'), 'logPayment'),
        "$SubTestName - verify logPayment was used.");
    ok(WasCalled($SUT, 'BlockMonitor'),
        "$SubTestName - verify block monitor was used.");

    ok(WasCalled($Servlet, 'locale'),
        "$SubTestName - verify locale was reseted.");
    ok(WasCalled($Servlet, 'languageID'),
        "$SubTestName - verify languageID was reseted.");
    ok(WasCalled($Servlet, 'currencyID'),
        "$SubTestName - verify currencyID was reseted.");

    my $aValidateDereferrerTokenParams = GetParametersFromMockifyCall($SUT, '_ValidateDereferrerToken');
    my ($Token, $PayLoad) = @{$aValidateDereferrerTokenParams};

    is($Token, 'xGHPdGpZ',
        "$SubTestName - verify token was params was used for request validation.");
    is($PayLoad, 'UnityShop&12233&445566-332211-221144',
        "$SubTestName - verify correct payload was used for request validation.");

    my $LineItemPayment = $self->memory->get('CustomerOrder')->container()->get('Payment');
    is($LineItemPayment->get('PaymentErrorCode'), 'Failed',
        "$SubTestName - verify payment error code was set.");

    is(GetParametersFromMockifyCall($Servlet, 'redirect')->[0], 'https://unity-instance.de:7000',
        "$SubTestName - verify servlet redirected to unity.");

    is(GetCallCount($Servlet->form(), 'executeFormError'), 0,
        "$SubTestName - verify no form error was executed.");

    return;
}

sub _prepareMocks {
    my $self = shift;

    my $UnityShop = $self->mockFactory->buildShop({
        'Alias' => 'UnityShop',
        'ID' => '123',
        'IsUnityRegistered' => 1
    });
    my $MultistepShop = $self->mockFactory->buildShop({
       'Alias' => 'MultistepShop',
       'ID' => '123',
       'IsUnityRegistered' => 0,
       'OrderProcess' => MULTISTEP_CHECKOUT
    });
    my $ClassicalShop = $self->mockFactory->buildShop({
       'Alias' => 'ClassicalShop',
       'ID' => '123',
       'IsUnityRegistered' => 0,
       'OrderProcess' => CLASSIC_CHECKOUT
    });

    my $PaymentMethodOrderBefore = $self->mockFactory->build(
        'DE_EPAGES::Payment::API::Object::PaymentMethod', {
            'DoPaymentOnBasket' => 0
        }
    );
    my $PaymentMethodOrderAfter = $self->mockFactory->build(
        'DE_EPAGES::Payment::API::Object::PaymentMethod', {
            'DoPaymentOnBasket' => 1
        }
    );

    my $OrderService = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::PayPalPlus::API::OrderService', []);

    my $Logger = DE_EPAGES::Test::Mock::Mockify->new( 'DE_EPAGES::PayPalPlus::API::Logger', [] );
    $Logger->mock('logPayment', undef, ['string', 'hashref']);

    my $LogContext = DE_EPAGES::Test::Mock::Mockify->new( 'DE_EPAGES::PayPalPlus::API::ContextLogger', [] );
    $LogContext->mock('logContext',
        sub {
            my $InternalSelf = shift;
            $InternalSelf->{'__ToCheck'}->{'_Params_logContext'} = \@_;
            my $FunctionPointer = $InternalSelf->{'__ToCheck'}->{'_Params_logContext'}[2];
            return $FunctionPointer->();
        }
    );

    $self->memory->add('LogContext', $LogContext->getMockObject());
    $self->memory->add('Logger', $Logger->getMockObject());
    $self->memory->add('UnityShop', $UnityShop->getMockObject());
    $self->memory->add('MultistepShop', $MultistepShop->getMockObject());
    $self->memory->add('ClassicalShop', $ClassicalShop->getMockObject());
    $self->memory->add('PaymentMethodOrderBefore', $PaymentMethodOrderBefore->getMockObject());
    $self->memory->add('PaymentMethodOrderAfter', $PaymentMethodOrderAfter->getMockObject());
    $self->memory->add('mock_OrderService', $OrderService);
    $self->memory->add('OrderService', $OrderService->getMockObject());
    $self->memory->add('CustomerOrder', undef);

    return;
}

sub _buildSUT {
    my $self = shift;
    my ($hParams) = @_;

    my $OrderService = $self->memory->get('mock_OrderService');
    $OrderService->mock('proccessOrder', $hParams->{'CanProcessOrder'}, ['hashref']);
    $OrderService->mock('getBasketOrOrder', $self->_buildCustomerOrder($hParams));
    $OrderService->mock('getErrorMessage', 'FailedToProcessOrder');
    $self->memory->replace('OrderService', $OrderService->getMockObject());

    my $SUT = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::PayPalPlus::UI::Shop', []);
    $SUT->mock('_LogWarning', undef, ['string']);
    $SUT->mock('_ValidateDereferrerToken', 1, ['string', 'string']);
    $SUT->mock('_createOrderService', $self->memory->get('OrderService'));
    $SUT->mock('_createLogContext', $self->memory->get('LogContext'));
    $SUT->mock('_createLogger', $self->memory->get('Logger'));
    $SUT->mock('_BuildShopUrl', 'http://payment-step.de', ['object', 'hashref', 'hashref', 'object']);
    $SUT->mock('BlockMonitor', undef, ['object']);

    return $SUT->getMockObject();
}

sub _buildServlet {
    my $self = shift;
    my ($hParams) = @_;

    my $Servlet = DE_EPAGES::Test::Mock::MockifyServlet->new($hParams->{'Shop'});
    $Servlet->addVars({
        'IsMobile' => $hParams->{'IsMobile'}
    });

    my $Form = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Presentation::API::Form', []);
    $Form->mock('executeFormError', undef, ['hashref']);

    $Servlet->mock('form', $Form->getMockObject());
    $Servlet->mock('locale', undef, [{'int' => 1}]);
    $Servlet->mock('languageID', undef, [{'int' => 1}]);
    $Servlet->mock('currencyID', undef, [{'int' => 1}]);
    $Servlet->mock('redirect', undef, ['string']);
    $Servlet->addParams({
        'PaymentLineItemGUID' => $hParams->{'PaymentLineItemGUID'},
        'ObjectID' => $hParams->{'ObjectID'},
        'DereferrerToken' => $hParams->{'DereferrerToken'}
    });

    return $Servlet->getMockObject();
}

sub _buildCustomerOrder {
    my $self = shift;
    my ($hAttributes) = @_;

    my $PaymentMethod = ($hAttributes->{'OrderBeforePayment'})
        ? $self->memory->get('PaymentMethodOrderBefore')
        : $self->memory->get('PaymentMethodOrderAfter');

    my $LineItemPayment = $self->mockFactory->buildLineItemPayment({
        'PaymentMethod' => $PaymentMethod,
        'PaymentErrorCode' => undef
    })->getMockObject();

    my $LineItemContainer = $self->mockFactory->buildLineItemContainer({
        'Payment' => $LineItemPayment
    })->getMockObject();

    my $CustomerOrder = $self->mockFactory->buildCustomerOrder({
        'Shop' => $hAttributes->{'Shop'},
        'Site' => $hAttributes->{'Shop'},
        'ID' => $hAttributes->{'ID'},
        'LineItemContainer' => $LineItemContainer,
        'WebUrlUnitySSL' => 'https://unity-instance.de:7000',
        'User' => 'Max Mustermann',
    });
    $self->memory->replace('CustomerOrder', $CustomerOrder->getMockObject());
    return $self->memory->get('CustomerOrder');
}

__PACKAGE__->runScript();