#========================================================================================
# package      DE_EPAGES::PayPalPlus::t::unit::UI_Shop_ViewPayPalPlusCancelPayment
# base         DE_EPAGES::Test::API::UnitTest::Test
#----------------------------------------------------------------------------------------
# description  unit test for DE_EPAGES::PayPalPlus::UI::Shop
#========================================================================================
package DE_EPAGES::PayPalPlus::t::unit::UI_Shop_ViewPayPalPlusCancelPayment;
use base DE_EPAGES::Test::API::UnitTest::Test;

use DE_EPAGES::Test::Mock::Mockify qw (
    WasCalled
    GetParametersFromMockifyCall
    GetCallCount
);
use DE_EPAGES::Test::Mock::MockifyServlet;
use DE_EPAGES::PayPalPlus::API::Constants qw (
    MULTISTEP_CHECKOUT
    CLASSIC_CHECKOUT
);
use Test::More;
use Test::Exception;
use strict;

sub test {
    my $self = shift;

    $self->_prepareMocks();

    $self->test_ViewPayPalPlusCancelPayment_canceled_multistep_before();
    $self->test_ViewPayPalPlusCancelPayment_canceled_multistep_after();
    $self->test_ViewPayPalPlusCancelPayment_failed_multistep_before();
    $self->test_ViewPayPalPlusCancelPayment_failed_multistep_after();

    $self->test_ViewPayPalPlusCancelPayment_canceled_classical_before();
    $self->test_ViewPayPalPlusCancelPayment_canceled_classical_after();
    $self->test_ViewPayPalPlusCancelPayment_failed_classical_before();
    $self->test_ViewPayPalPlusCancelPayment_failed_classical_after();

    $self->test_ViewPayPalPlusCancelPayment_canceled_mobile_before();
    $self->test_ViewPayPalPlusCancelPayment_canceled_mobile_after();
    $self->test_ViewPayPalPlusCancelPayment_failed_mobile_before();
    $self->test_ViewPayPalPlusCancelPayment_failed_mobile_after();

    $self->test_ViewPayPalPlusCancelPayment_canceled_unity_before();
    $self->test_ViewPayPalPlusCancelPayment_failed_unity_before();

    $self->test_ViewPayPalPlusCancelPayment_missing_guid();

    done_testing();
}

sub test_ViewPayPalPlusCancelPayment_canceled_mobile_before {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    ## GIVEN
    my $SUT = $self->_buildSUT({
        'OrderBeforePayment' => 1,
        'ID' => '20304',
        'Shop' => $self->memory->get('MultistepShop')
    });
    my $Servlet = $self->_buildServlet({
        'IsMobile' => 1,
        'Shop' => $self->memory->get('MultistepShop'),
        'ppp_msg' => undef,
        'PaymentLineItemGUID' => '445566-332211-221144',
    });

    ## WHEN
    $SUT->ViewPayPalPlusCancelPayment($Servlet);

    ## THEN
    ok(WasCalled($self->memory->get('LogContext'), 'logContext'),
        "$SubTestName - verify logContext was used.");
    ok(WasCalled($self->memory->get('Logger'), 'logPayment'),
        "$SubTestName - verify logPayment was used.");

    ok(WasCalled($self->memory->get('PaymentLineItemService'), 'paymentCanceled'),
        "$SubTestName - verify payment transaction status was set to cancle.");
    ok(WasCalled($SUT, '_LogWarning'),
        "$SubTestName - verify a error message was logged.");

    ok(WasCalled($Servlet->form(), 'executeFormError'),
        "$SubTestName - verify form error was executed.");
    is_deeply(GetParametersFromMockifyCall($Servlet->form(), 'executeFormError')->[0], {
        'Name'        => 'PayPalPlus_PaymentFailed',
        'Reason'      => 'Canceled',
        'ViewAction'  => 'ViewPaymentPayPalPlusResult',
        'ObjectID'    => '20304'
    }, "$SubTestName - verify the form error has expected attributes.");

    return;
}

sub test_ViewPayPalPlusCancelPayment_canceled_mobile_after {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    ## GIVEN
    my $SUT = $self->_buildSUT({
        'OrderBeforePayment' => 0,
        'ID' => '20304',
        'Shop' => $self->memory->get('MultistepShop')
    });
    my $Servlet = $self->_buildServlet({
        'IsMobile' => 1,
        'Shop' => $self->memory->get('MultistepShop'),
        'ppp_msg' => undef,
        'PaymentLineItemGUID' => '445566-332211-221144',
    });

    ## WHEN
    $SUT->ViewPayPalPlusCancelPayment($Servlet);

    ## THEN
    ok(WasCalled($self->memory->get('LogContext'), 'logContext'),
        "$SubTestName - verify logContext was used.");
    ok(WasCalled($self->memory->get('Logger'), 'logPayment'),
        "$SubTestName - verify logPayment was used.");

    ok(WasCalled($self->memory->get('PaymentLineItemService'), 'paymentCanceled'),
        "$SubTestName - verify payment transaction status was set to cancle.");
    ok(WasCalled($SUT, '_LogWarning'),
        "$SubTestName - verify a error message was logged.");

    my $aParameters = GetParametersFromMockifyCall($SUT, '_BuildShopUrl');
    my ($Object, $hParams, $hOptions) = @{$aParameters}; ## no critic (ProhibitUnusedVariablesEnhanced)

    is_deeply($hParams, {},
        "$SubTestName - verify the expected params where used to build the redirect url.");
    is_deeply($hOptions, {
        'Type' => 'sf/mobile',
        'UseSSL' => 1,
        'AddToken' => 1
    }, "$SubTestName - verify the expected options where used to build the redirect url.");
    ok(WasCalled($Servlet, 'redirect'),
        "$SubTestName - verify the servlet redirect was called.");

    is(GetCallCount($Servlet->form(), 'executeFormError'), 0,
        "$SubTestName - verify no form error was executed.");

    return;
}

sub test_ViewPayPalPlusCancelPayment_failed_mobile_before {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    ## GIVEN
    my $SUT = $self->_buildSUT({
        'OrderBeforePayment' => 1,
        'ID' => '20304',
        'Shop' => $self->memory->get('MultistepShop')
    });
    my $Servlet = $self->_buildServlet({
        'IsMobile' => 1,
        'Shop' => $self->memory->get('MultistepShop'),
        'ppp_msg' => 1,
        'PaymentLineItemGUID' => '445566-332211-221144',
    });

    ## WHEN
    $SUT->ViewPayPalPlusCancelPayment($Servlet);

    ## THEN
    ok(WasCalled($self->memory->get('LogContext'), 'logContext'),
        "$SubTestName - verify logContext was used.");
    ok(WasCalled($self->memory->get('Logger'), 'logPayment'),
        "$SubTestName - verify logPayment was used.");

    ok(WasCalled($self->memory->get('PaymentLineItemService'), 'paymentCanceled'),
        "$SubTestName - verify payment transaction status was set to cancle.");
    ok(WasCalled($SUT, '_LogWarning'),
        "$SubTestName - verify a error message was logged.");

    ok(WasCalled($Servlet->form(), 'executeFormError'),
        "$SubTestName - verify form error was executed.");
    is_deeply(GetParametersFromMockifyCall($Servlet->form(), 'executeFormError')->[0], {
        'Name'        => 'PayPalPlus_PaymentFailed',
        'Reason'      => 'Failed',
        'ViewAction'  => 'ViewPaymentPayPalPlusResult',
        'ObjectID'    => '20304'
    }, "$SubTestName - verify the form error has expected attributes.");

    return;
}

sub test_ViewPayPalPlusCancelPayment_failed_mobile_after {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    ## GIVEN
    my $SUT = $self->_buildSUT({
        'OrderBeforePayment' => 0,
        'ID' => '20304',
        'Shop' => $self->memory->get('MultistepShop')
    });
    my $Servlet = $self->_buildServlet({
        'IsMobile' => 1,
        'Shop' => $self->memory->get('MultistepShop'),
        'ppp_msg' => 1,
        'PaymentLineItemGUID' => '445566-332211-221144',
    });

    ## WHEN
    $SUT->ViewPayPalPlusCancelPayment($Servlet);

    ## THEN
    ok(WasCalled($self->memory->get('LogContext'), 'logContext'),
        "$SubTestName - verify logContext was used.");
    ok(WasCalled($self->memory->get('Logger'), 'logPayment'),
        "$SubTestName - verify logPayment was used.");

    ok(WasCalled($self->memory->get('PaymentLineItemService'), 'paymentCanceled'),
        "$SubTestName - verify payment transaction status was set to cancle.");
    ok(WasCalled($SUT, '_LogWarning'),
        "$SubTestName - verify a error message was logged.");

    my $aParameters = GetParametersFromMockifyCall($SUT, '_BuildShopUrl');
    my ($Object, $hParams, $hOptions) = @{$aParameters}; ## no critic (ProhibitUnusedVariablesEnhanced)

    is_deeply($hParams, {
        'ppp_msg' => 1
    },
        "$SubTestName - verify the expected params where used to build the redirect url.");
    is_deeply($hOptions, {
        'Type' => 'sf/mobile',
        'UseSSL' => 1,
        'AddToken' => 1
    }, "$SubTestName - verify the expected options where used to build the redirect url.");
    ok(WasCalled($Servlet, 'redirect'),
        "$SubTestName - verify the servlet redirect was called.");

    is(GetCallCount($Servlet->form(), 'executeFormError'), 0,
        "$SubTestName - verify no form error was executed.");

    return;
}

sub test_ViewPayPalPlusCancelPayment_failed_classical_before {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    ## GIVEN
    my $SUT = $self->_buildSUT({
        'OrderBeforePayment' => 1,
        'ID' => '20304',
        'Shop' => $self->memory->get('ClassicalShop')
    });
    my $Servlet = $self->_buildServlet({
        'IsMobile' => 0,
        'Shop' => $self->memory->get('ClassicalShop'),
        'ppp_msg' => 1,
        'PaymentLineItemGUID' => '445566-332211-221144',
    });

    ## WHEN
    $SUT->ViewPayPalPlusCancelPayment($Servlet);

    ## THEN
    ok(WasCalled($self->memory->get('LogContext'), 'logContext'),
        "$SubTestName - verify logContext was used.");
    ok(WasCalled($self->memory->get('Logger'), 'logPayment'),
        "$SubTestName - verify logPayment was used.");

    ok(WasCalled($self->memory->get('PaymentLineItemService'), 'paymentCanceled'),
        "$SubTestName - verify payment transaction status was set to cancle.");
    ok(WasCalled($SUT, '_LogWarning'),
        "$SubTestName - verify a error message was logged.");

    ok(WasCalled($Servlet->form(), 'executeFormError'),
        "$SubTestName - verify form error was executed.");
    is_deeply(GetParametersFromMockifyCall($Servlet->form(), 'executeFormError')->[0], {
        'Name'        => 'PayPalPlus_PaymentFailed',
        'Reason'      => 'Failed',
        'ViewAction'  => 'ViewPaymentPayPalPlusResult',
        'ObjectID'    => '20304'
    }, "$SubTestName - verify the form error has expected attributes.");

    return;
}

sub test_ViewPayPalPlusCancelPayment_failed_classical_after {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    ## GIVEN
    my $SUT = $self->_buildSUT({
        'OrderBeforePayment' => 0,
        'ID' => '20304',
        'Shop' => $self->memory->get('ClassicalShop')
    });
    my $Servlet = $self->_buildServlet({
        'IsMobile' => 0,
        'Shop' => $self->memory->get('ClassicalShop'),
        'ppp_msg' => 1,
        'PaymentLineItemGUID' => '445566-332211-221144',
    });

    ## WHEN
    $SUT->ViewPayPalPlusCancelPayment($Servlet);

    ## THEN
    ok(WasCalled($self->memory->get('LogContext'), 'logContext'),
        "$SubTestName - verify logContext was used.");
    ok(WasCalled($self->memory->get('Logger'), 'logPayment'),
        "$SubTestName - verify logPayment was used.");

    ok(WasCalled($self->memory->get('PaymentLineItemService'), 'paymentCanceled'),
        "$SubTestName - verify payment transaction status was set to cancle.");
    ok(WasCalled($SUT, '_LogWarning'),
        "$SubTestName - verify a error message was logged.");

    my $aParameters = GetParametersFromMockifyCall($SUT, '_BuildShopUrl');
    my ($Object, $hParams, $hOptions) = @{$aParameters}; ## no critic (ProhibitUnusedVariablesEnhanced)

    is_deeply($hParams, {
        'ppp_msg' => 1
    }, "$SubTestName - verify the expected params where used to build the redirect url.");
    is_deeply($hOptions, {
        'Type' => 'sf/mobile',
        'UseSSL' => 1,
        'AddToken' => 1
    }, "$SubTestName - verify the expected options where used to build the redirect url.");
    ok(WasCalled($Servlet, 'redirect'),
        "$SubTestName - verify the servlet redirect was called.");

    is(GetCallCount($Servlet->form(), 'executeFormError'), 0,
        "$SubTestName - verify no form error was executed.");

    return;
}

sub test_ViewPayPalPlusCancelPayment_canceled_classical_before {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    ## GIVEN
    my $SUT = $self->_buildSUT({
        'OrderBeforePayment' => 1,
        'ID' => '20304',
        'Shop' => $self->memory->get('ClassicalShop')
    });
    my $Servlet = $self->_buildServlet({
        'IsMobile' => 0,
        'Shop' => $self->memory->get('ClassicalShop'),
        'ppp_msg' => undef,
        'PaymentLineItemGUID' => '445566-332211-221144',
    });

    ## WHEN
    $SUT->ViewPayPalPlusCancelPayment($Servlet);

    ## THEN
    ok(WasCalled($self->memory->get('LogContext'), 'logContext'),
        "$SubTestName - verify logContext was used.");
    ok(WasCalled($self->memory->get('Logger'), 'logPayment'),
        "$SubTestName - verify logPayment was used.");

    ok(WasCalled($self->memory->get('PaymentLineItemService'), 'paymentCanceled'),
        "$SubTestName - verify payment transaction status was set to cancle.");
    ok(WasCalled($SUT, '_LogWarning'),
        "$SubTestName - verify a error message was logged.");

    ok(WasCalled($Servlet->form(), 'executeFormError'),
        "$SubTestName - verify form error was executed.");
    is_deeply(GetParametersFromMockifyCall($Servlet->form(), 'executeFormError')->[0], {
        'Name'        => 'PayPalPlus_PaymentFailed',
        'Reason'      => 'Canceled',
        'ViewAction'  => 'ViewPaymentPayPalPlusResult',
        'ObjectID'    => '20304'
    }, "$SubTestName - verify the form error has expected attributes.");

    return;
}

sub test_ViewPayPalPlusCancelPayment_canceled_classical_after {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    ## GIVEN
    my $SUT = $self->_buildSUT({
        'OrderBeforePayment' => 0,
        'ID' => '20304',
        'Shop' => $self->memory->get('ClassicalShop')
    });
    my $Servlet = $self->_buildServlet({
        'IsMobile' => 0,
        'Shop' => $self->memory->get('ClassicalShop'),
        'ppp_msg' => undef,
        'PaymentLineItemGUID' => '445566-332211-221144',
    });

    ## WHEN
    $SUT->ViewPayPalPlusCancelPayment($Servlet);

    ## THEN
    ok(WasCalled($self->memory->get('LogContext'), 'logContext'),
        "$SubTestName - verify logContext was used.");
    ok(WasCalled($self->memory->get('Logger'), 'logPayment'),
        "$SubTestName - verify logPayment was used.");

    ok(WasCalled($self->memory->get('PaymentLineItemService'), 'paymentCanceled'),
        "$SubTestName - verify payment transaction status was set to cancle.");
    ok(WasCalled($SUT, '_LogWarning'),
        "$SubTestName - verify a error message was logged.");

    my $aParameters = GetParametersFromMockifyCall($SUT, '_BuildShopUrl');
    my ($Object, $hParams, $hOptions) = @{$aParameters}; ## no critic (ProhibitUnusedVariablesEnhanced)

    is_deeply($hParams, {},
        "$SubTestName - verify the expected params where used to build the redirect url.");
    is_deeply($hOptions, {
        'Type' => 'sf/mobile',
        'UseSSL' => 1,
        'AddToken' => 1
    }, "$SubTestName - verify the expected options where used to build the redirect url.");
    ok(WasCalled($Servlet, 'redirect'),
        "$SubTestName - verify the servlet redirect was called.");

    is(GetCallCount($Servlet->form(), 'executeFormError'), 0,
        "$SubTestName - verify no form error was executed.");

    return;
}

sub test_ViewPayPalPlusCancelPayment_failed_multistep_before {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    ## GIVEN
    my $SUT = $self->_buildSUT({
        'OrderBeforePayment' => 1,
        'ID' => '20304',
        'Shop' => $self->memory->get('MultistepShop')
    });
    my $Servlet = $self->_buildServlet({
        'IsMobile' => 0,
        'Shop' => $self->memory->get('MultistepShop'),
        'ppp_msg' => 1,
        'PaymentLineItemGUID' => '445566-332211-221144',
    });

    ## WHEN
    $SUT->ViewPayPalPlusCancelPayment($Servlet);

    ## THEN
    ok(WasCalled($self->memory->get('LogContext'), 'logContext'),
        "$SubTestName - verify logContext was used.");
    ok(WasCalled($self->memory->get('Logger'), 'logPayment'),
        "$SubTestName - verify logPayment was used.");

    ok(WasCalled($self->memory->get('PaymentLineItemService'), 'paymentCanceled'),
        "$SubTestName - verify payment transaction status was set to cancle.");
    ok(WasCalled($SUT, '_LogWarning'),
        "$SubTestName - verify a error message was logged.");

    ok(WasCalled($Servlet->form(), 'executeFormError'),
        "$SubTestName - verify form error was executed.");
    is_deeply(GetParametersFromMockifyCall($Servlet->form(), 'executeFormError')->[0], {
        'Name'        => 'PayPalPlus_PaymentFailed',
        'Reason'      => 'Failed',
        'ViewAction'  => 'ViewPaymentPayPalPlusResult',
        'ObjectID'    => '20304'
    }, "$SubTestName - verify the form error has expected attributes.");

    return;
}

sub test_ViewPayPalPlusCancelPayment_failed_multistep_after {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    ## GIVEN
    my $SUT = $self->_buildSUT({
        'OrderBeforePayment' => 0,
        'ID' => '20304',
        'Shop' => $self->memory->get('MultistepShop')
    });
    my $Servlet = $self->_buildServlet({
        'IsMobile' => 0,
        'Shop' => $self->memory->get('MultistepShop'),
        'ppp_msg' => 1,
        'PaymentLineItemGUID' => '445566-332211-221144',
    });

    ## WHEN
    $SUT->ViewPayPalPlusCancelPayment($Servlet);

    ## THEN
    ok(WasCalled($self->memory->get('LogContext'), 'logContext'),
        "$SubTestName - verify logContext was used.");
    ok(WasCalled($self->memory->get('Logger'), 'logPayment'),
        "$SubTestName - verify logPayment was used.");

    ok(WasCalled($self->memory->get('PaymentLineItemService'), 'paymentCanceled'),
        "$SubTestName - verify payment transaction status was set to cancle.");
    ok(WasCalled($SUT, '_LogWarning'),
        "$SubTestName - verify a error message was logged.");

    my $aParameters = GetParametersFromMockifyCall($SUT, '_BuildShopUrl');
    my ($Object, $hParams, $hOptions) = @{$aParameters}; ## no critic (ProhibitUnusedVariablesEnhanced)

    is_deeply($hParams, {
        'ViewAction' => 'ViewMultiCheckoutPayment',
        'ppp_msg' => 1
    }, "$SubTestName - verify the expected params where used to build the redirect url.");
    is_deeply($hOptions, {
        'Type' => 'sf/mobile',
        'UseSSL' => 1,
        'AddToken' => 1
    }, "$SubTestName - verify the expected options where used to build the redirect url.");
    ok(WasCalled($Servlet, 'redirect'),
        "$SubTestName - verify the servlet redirect was called.");

    is(GetCallCount($Servlet->form(), 'executeFormError'), 0,
        "$SubTestName - verify no form error was executed.");

    return;
}

sub test_ViewPayPalPlusCancelPayment_canceled_multistep_before {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    ## GIVEN
    my $SUT = $self->_buildSUT({
        'OrderBeforePayment' => 1,
        'ID' => '20304',
        'Shop' => $self->memory->get('MultistepShop')
    });
    my $Servlet = $self->_buildServlet({
        'IsMobile' => 0,
        'Shop' => $self->memory->get('MultistepShop'),
        'ppp_msg' => undef,
        'PaymentLineItemGUID' => '445566-332211-221144',
    });

    ## WHEN
    $SUT->ViewPayPalPlusCancelPayment($Servlet);

    ## THEN
    ok(WasCalled($self->memory->get('LogContext'), 'logContext'),
        "$SubTestName - verify logContext was used.");
    ok(WasCalled($self->memory->get('Logger'), 'logPayment'),
        "$SubTestName - verify logPayment was used.");

    ok(WasCalled($self->memory->get('PaymentLineItemService'), 'paymentCanceled'),
        "$SubTestName - verify payment transaction status was set to cancle.");
    ok(WasCalled($SUT, '_LogWarning'),
        "$SubTestName - verify a error message was logged.");

    ok(WasCalled($Servlet->form(), 'executeFormError'),
        "$SubTestName - verify form error was executed.");
    is_deeply(GetParametersFromMockifyCall($Servlet->form(), 'executeFormError')->[0], {
        'Name'        => 'PayPalPlus_PaymentFailed',
        'Reason'      => 'Canceled',
        'ViewAction'  => 'ViewPaymentPayPalPlusResult',
        'ObjectID'    => '20304'
    }, "$SubTestName - verify the form error has expected attributes.");

    return;
}

sub test_ViewPayPalPlusCancelPayment_canceled_multistep_after {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    ## GIVEN
    my $SUT = $self->_buildSUT({
        'OrderBeforePayment' => 0,
        'ID' => '20304',
        'Shop' => $self->memory->get('MultistepShop')
    });
    my $Servlet = $self->_buildServlet({
        'IsMobile' => 0,
        'Shop' => $self->memory->get('MultistepShop'),
        'ppp_msg' => undef,
        'PaymentLineItemGUID' => '445566-332211-221144',
    });

    ## WHEN
    $SUT->ViewPayPalPlusCancelPayment($Servlet);

    ## THEN
    ok(WasCalled($self->memory->get('LogContext'), 'logContext'),
        "$SubTestName - verify logContext was used.");
    ok(WasCalled($self->memory->get('Logger'), 'logPayment'),
        "$SubTestName - verify logPayment was used.");

    ok(WasCalled($self->memory->get('PaymentLineItemService'), 'paymentCanceled'),
        "$SubTestName - verify payment transaction status was set to cancle.");
    ok(WasCalled($SUT, '_LogWarning'),
        "$SubTestName - verify a error message was logged.");

    my $aParameters = GetParametersFromMockifyCall($SUT, '_BuildShopUrl');
    my ($Object, $hParams, $hOptions) = @{$aParameters}; ## no critic (ProhibitUnusedVariablesEnhanced)

    is_deeply($hParams, {
        'ViewAction' => 'ViewMultiCheckoutPayment'
    }, "$SubTestName - verify the expected params where used to build the redirect url.");
    is_deeply($hOptions, {
        'Type' => 'sf/mobile',
        'UseSSL' => 1,
        'AddToken' => 1
    }, "$SubTestName - verify the expected options where used to build the redirect url.");
    ok(WasCalled($Servlet, 'redirect'),
        "$SubTestName - verify the servlet redirect was called.");

    is(GetCallCount($Servlet->form(), 'executeFormError'), 0,
        "$SubTestName - verify no form error was executed.");

    return;
}

sub test_ViewPayPalPlusCancelPayment_canceled_unity_before {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    ## GIVEN
    my $SUT = $self->_buildSUT({
        'OrderBeforePayment' => 1,
        'ID' => '20304',
        'Shop' => $self->memory->get('UnityShop')
    });
    my $Servlet = $self->_buildServlet({
        'IsMobile' => 0,
        'Shop' => $self->memory->get('UnityShop'),
        'ppp_msg' => undef,
        'PaymentLineItemGUID' => '445566-332211-221144',
    });

    ## WHEN
    $SUT->ViewPayPalPlusCancelPayment($Servlet);

    ## THEN
    ok(WasCalled($self->memory->get('LogContext'), 'logContext'),
        "$SubTestName - verify logContext was used.");
    ok(WasCalled($self->memory->get('Logger'), 'logPayment'),
        "$SubTestName - verify logPayment was used.");

    ok(WasCalled($self->memory->get('PaymentLineItemService'), 'paymentCanceled'),
        "$SubTestName - verify payment transaction status was set to cancle.");
    ok(WasCalled($SUT, '_LogWarning'),
        "$SubTestName - verify a error message was logged.");

    my $LineItemPayment = $self->memory->get('CustomerOrder')->container()->get('Payment');
    is($LineItemPayment->get('PaymentErrorCode'), 'Canceled',
        "$SubTestName - verify payment error code was set.");

    is(GetParametersFromMockifyCall($Servlet, 'redirect')->[0], 'https://unity-instance.de:7000',
        "$SubTestName - verify servlet redirected to unity.");

    is(GetCallCount($Servlet->form(), 'executeFormError'), 0,
        "$SubTestName - verify no form error was executed.");

    return;
}

sub test_ViewPayPalPlusCancelPayment_failed_unity_before {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    ## GIVEN
    my $SUT = $self->_buildSUT({
        'OrderBeforePayment' => 1,
        'ID' => '20304',
        'Shop' => $self->memory->get('UnityShop')
    });
    my $Servlet = $self->_buildServlet({
        'IsMobile' => 0,
        'Shop' => $self->memory->get('UnityShop'),
        'ppp_msg' => 1,
        'PaymentLineItemGUID' => '445566-332211-221144',
    });

    ## WHEN
    $SUT->ViewPayPalPlusCancelPayment($Servlet);

    ## THEN
    ok(WasCalled($self->memory->get('LogContext'), 'logContext'),
        "$SubTestName - verify logContext was used.");
    ok(WasCalled($self->memory->get('Logger'), 'logPayment'),
        "$SubTestName - verify logPayment was used.");

    ok(WasCalled($self->memory->get('PaymentLineItemService'), 'paymentCanceled'),
        "$SubTestName - verify payment transaction status was set to cancle.");
    ok(WasCalled($SUT, '_LogWarning'),
        "$SubTestName - verify a error message was logged.");

    my $LineItemPayment = $self->memory->get('CustomerOrder')->container()->get('Payment');
    is($LineItemPayment->get('PaymentErrorCode'), 'Failed',
        "$SubTestName - verify payment error code was set.");

    is(GetParametersFromMockifyCall($Servlet, 'redirect')->[0], 'https://unity-instance.de:7000',
        "$SubTestName - verify servlet redirected to unity.");

    is(GetCallCount($Servlet->form(), 'executeFormError'), 0,
        "$SubTestName - verify no form error was executed.");

    return;
}

sub test_ViewPayPalPlusCancelPayment_missing_guid {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    ## GIVEN
    my $SUT = $self->_buildSUT({
        'OrderBeforePayment' => 1,
        'ID' => '20304',
        'Shop' => $self->memory->get('MultistepShop')
    });
    my $Servlet = $self->_buildServlet({
        'IsMobile' => 1,
        'Shop' => $self->memory->get('MultistepShop'),
        'ppp_msg' => undef,
        'PaymentLineItemGUID' => undef,
    });

    ## WHEN
    $SUT->ViewPayPalPlusCancelPayment($Servlet);

    ## THEN
    ok(WasCalled($self->memory->get('LogContext'), 'logContext'),
        "$SubTestName - verify logContext was used.");
    ok(WasCalled($self->memory->get('Logger'), 'logPayment'),
        "$SubTestName - verify logPayment was used.");

    ok(WasCalled($Servlet->form(), 'executeFormError'),
        "$SubTestName - verify form error was executed.");
    is_deeply(GetParametersFromMockifyCall($Servlet->form(), 'executeFormError')->[0], {
        'Name'        => 'PayPalPlus_PaymentFailed',
        'Reason'      => 'BadCallback_NoPaymentGUID',
        'ViewAction'  => 'ViewPaymentPayPalPlusResult'
    }, "$SubTestName - verify the form error has expected attributes.");

    return;
}

sub _prepareMocks {
    my $self = shift;

    my $UnityShop = $self->mockFactory->buildShop({
        'Alias' => 'UnityShop',
        'ID' => '123',
        'IsUnityRegistered' => 1
    });
    my $MultistepShop = $self->mockFactory->buildShop({
       'Alias' => 'MultistepShop',
       'ID' => '123',
       'IsUnityRegistered' => 0,
       'OrderProcess' => MULTISTEP_CHECKOUT
    });
    my $ClassicalShop = $self->mockFactory->buildShop({
       'Alias' => 'ClassicalShop',
       'ID' => '123',
       'IsUnityRegistered' => 0,
       'OrderProcess' => CLASSIC_CHECKOUT
    });

    my $PaymentMethodOrderBefore = $self->mockFactory->build(
        'DE_EPAGES::Payment::API::Object::PaymentMethod', {
            'DoPaymentOnBasket' => 0
        }
    );
    my $PaymentMethodOrderAfter = $self->mockFactory->build(
        'DE_EPAGES::Payment::API::Object::PaymentMethod', {
            'DoPaymentOnBasket' => 1
        }
    );

    my $PaymentLineItemService = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::PayPalPlus::API::PaymentLineItemService', []);
    $PaymentLineItemService->addMockWithConditionalReturn('canLoadByGuid', 1, [{ 'string' => '445566-332211-221144' }]);
    $PaymentLineItemService->addMockWithConditionalReturn('canLoadByGuid', 0, [{ 'string' => undef }]);
    $PaymentLineItemService->mock('paymentCanceled', undef);
    $PaymentLineItemService->mock('paymentFailed', undef);

    my $Logger = DE_EPAGES::Test::Mock::Mockify->new( 'DE_EPAGES::PayPalPlus::API::Logger', [] );
    $Logger->mock('logPayment', undef, ['string', 'hashref']);

    my $LogContext = DE_EPAGES::Test::Mock::Mockify->new( 'DE_EPAGES::PayPalPlus::API::ContextLogger', [] );
    $LogContext->mock('logContext',
        sub {
            my $InternalSelf = shift;
            $InternalSelf->{'__ToCheck'}->{'_Params_logContext'} = \@_;
            my $FunctionPointer = $InternalSelf->{'__ToCheck'}->{'_Params_logContext'}[2];
            return $FunctionPointer->();
        }
    );

    $self->memory->add('LogContext', $LogContext->getMockObject());
    $self->memory->add('Logger', $Logger->getMockObject());
    $self->memory->add('UnityShop', $UnityShop->getMockObject());
    $self->memory->add('MultistepShop', $MultistepShop->getMockObject());
    $self->memory->add('ClassicalShop', $ClassicalShop->getMockObject());
    $self->memory->add('PaymentMethodOrderBefore', $PaymentMethodOrderBefore->getMockObject());
    $self->memory->add('PaymentMethodOrderAfter', $PaymentMethodOrderAfter->getMockObject());
    $self->memory->add('mock_PaymentLineItemService', $PaymentLineItemService);
    $self->memory->add('PaymentLineItemService', $PaymentLineItemService->getMockObject());
    $self->memory->add('CustomerOrder', undef);

    return;
}

sub _buildSUT {
    my $self = shift;
    my ($hParams) = @_;

    my $PaymentLineItemService = $self->memory->get('mock_PaymentLineItemService');
    $PaymentLineItemService->mock('getBasketOrOrder', $self->_buildCustomerOrder($hParams));
    $self->memory->replace('PaymentLineItemService', $PaymentLineItemService->getMockObject());

    my $SUT = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::PayPalPlus::UI::Shop', []);
    $SUT->mock('_LogWarning', undef, ['string']);
    $SUT->addMockWithConditionalReturn('_ValidateDereferrerToken', 1, [
        { 'string' => 'f0d24af0e' },
        { 'string' => $hParams->{'Shop'}->alias().'&20505&445566-332211-221144' }
    ]);
    $SUT->addMockWithConditionalReturn('_ValidateDereferrerToken', 0, [
        { 'string' => 'f0d24af0e' },
        { 'string' => $hParams->{'Shop'}->alias().'&20505&' }
    ]);
    $SUT->mock('_createPaymentLineItemService', $self->memory->get('PaymentLineItemService'));
    $SUT->mock('_createLogContext', $self->memory->get('LogContext'));
    $SUT->mock('_createLogger', $self->memory->get('Logger'));
    $SUT->mock('_BuildShopUrl', 'http://payment-step.de', ['object', 'hashref', 'hashref', 'object']);

    return $SUT->getMockObject();
}

sub _buildServlet {
    my $self = shift;
    my ($hParams) = @_;

    my $Servlet = DE_EPAGES::Test::Mock::MockifyServlet->new($hParams->{'Shop'});
    $Servlet->addVars({
        'IsMobile' => $hParams->{'IsMobile'}
    });

    my $Form = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Presentation::API::Form', []);
    $Form->mock('executeFormError', undef, ['hashref']);

    $Servlet->mock('form', $Form->getMockObject());
    $Servlet->mock('locale', undef, [{'int' => 1}]);
    $Servlet->mock('languageID', undef, [{'int' => 1}]);
    $Servlet->mock('currencyID', undef, [{'int' => 1}]);
    $Servlet->mock('redirect', undef, ['string']);
    $Servlet->addParams( {
        'DereferrerToken' => 'f0d24af0e',
        'PaymentLineItemGUID' => $hParams->{'PaymentLineItemGUID'},
        'ObjectID' => '20505',
        'ppp_msg' => $hParams->{'ppp_msg'}
    });

    return $Servlet->getMockObject();
}

sub _buildCustomerOrder {
    my $self = shift;
    my ($hAttributes) = @_;

    my $PaymentMethod = ($hAttributes->{'OrderBeforePayment'})
        ? $self->memory->get('PaymentMethodOrderBefore')
        : $self->memory->get('PaymentMethodOrderAfter');

    my $LineItemPayment = $self->mockFactory->buildLineItemPayment({
        'PaymentMethod' => $PaymentMethod,
        'PaymentErrorCode' => undef
    })->getMockObject();

    my $LineItemContainer = $self->mockFactory->buildLineItemContainer({
        'Payment' => $LineItemPayment
    })->getMockObject();

    my $CustomerOrder = $self->mockFactory->buildCustomerOrder({
        'Shop' => $hAttributes->{'Shop'},
        'Site' => $hAttributes->{'Shop'},
        'ID' => $hAttributes->{'ID'},
        'LineItemContainer' => $LineItemContainer,
        'WebUrlUnitySSL' => 'https://unity-instance.de:7000',
        'User' => 'Max Mustermann',
    });
    $self->memory->replace('CustomerOrder', $CustomerOrder->getMockObject());
    return $self->memory->get('CustomerOrder');
}

__PACKAGE__->runScript();